!---------------------------------------------------------------------------------------
! FLEXINVERT inequality constraint
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Guide to use:
!!   - applies an inequality constraint to state variables found using
!!     Bayesian inversion with Gaussian PDF and is equivalent to solving the
!!     problem with a truncated Gaussian PDF
!!   - requires inversion output from FLEXINVERT 
!---------------------------------------------------------------------------------------

program main

  use mod_var

  implicit none

  integer :: ierr

  !---------------------------------------------------------------
  ! initialization 
  !---------------------------------------------------------------

  call readinitial()

  open(logid,file=trim(dirout)//trim(logfile),status='replace',action='write',iostat=ierr)

  !---------------------------------------------------------------
  ! read inputs
  !---------------------------------------------------------------
 
  call readinput()

  !---------------------------------------------------------------
  ! calculate constraint selector matrix
  !---------------------------------------------------------------

  call calcselector()

  !---------------------------------------------------------------
  ! bayesian inversion with inequality constraint
  !---------------------------------------------------------------

  call bayesian()

  !---------------------------------------------------------------
  ! map fluxes back to flexpart grid
  !---------------------------------------------------------------

  ! re-grid posterior emissions on fine grid
  ! if scaleflx = true : uses spatial distribution of prior fluxes

  allocate(emispost(nlon,nlat,ntime))

  call regridemissions(x,xb,emispost)

  !---------------------------------------------------------------
  ! save output
  !---------------------------------------------------------------
  
  call save()

  close(logid)

end program main

