!---------------------------------------------------------------------------------------
! FLEXINVERT : readinput
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Reads input data from inversion 
!---------------------------------------------------------------------------------------

subroutine readinput()

  use mod_var

  implicit none

  include '/usr/include/netcdf.inc'

  integer :: i,nid,ierr,xid,yid,zid,nx,ny,nz,varid
  real, dimension(:,:), allocatable :: tmp

  !---------------------------------------------------------------
  ! grid definition
  !---------------------------------------------------------------

  ! open netcdf file
  write(logid,*) 'Reading nbox_xy.nc'
  ierr=nf_open(trim(dirinvert)//'nbox_xy.nc',NF_NOWRITE,nid)
  if(ierr.ne.0) then
    write(logid,*) 'Error opening nbox_xy.nc'
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'dimx',xid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: dimx'
  endif
  ierr=nf_inq_dimid(nid,'dimy',yid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: dimy'
  endif

  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,nxgrid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  ierr=nf_inq_dimlen(nid,yid,nygrid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif

  allocate(nbox_xy(nxgrid,nygrid))
  allocate(tmp(nxgrid,nygrid))

  ! read variable
  ierr=nf_inq_varid(nid,'nbox_xy',varid)
  ierr=nf_get_var_real(nid,varid,tmp)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading nbox_xy'
  endif
  nbox_xy=int(tmp)
  nbox=maxval(nbox_xy)

  ierr=nf_close(nid)

  deallocate(tmp)

  !---------------------------------------------------------------
  ! grid box area
  !---------------------------------------------------------------

  allocate(area_box(nbox))

  open(100,file=trim(dirinvert)//'area_box.txt',action='read',status='old',iostat=ierr)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading area_box.txt'
  endif
  do i=1,nbox
    read(100,*) area_box(i)
  end do
  close(100)

  !---------------------------------------------------------------
  ! prior flux at fine resolution
  !---------------------------------------------------------------

  ! open netcdf file
  write(logid,*) 'Reading analysis.nc'
  ierr=nf_open(trim(dirinvert)//'analysis.nc',NF_NOWRITE,nid)
  if(ierr.ne.0) then
    write(logid,*) 'Error opening analysis.nc'
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'longitude',xid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: longitude'
  endif
  ierr=nf_inq_dimid(nid,'latitude',yid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: latitude'
  endif  
  ierr=nf_inq_dimid(nid,'time',zid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: time'
  endif

  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,nlon)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  ierr=nf_inq_dimlen(nid,yid,nlat)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  ierr=nf_inq_dimlen(nid,zid,ntime)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif

  allocate(lon(nlon))
  allocate(lat(nlat))
  allocate(time(ntime))
  allocate(emisprior(nlon,nlat,ntime))

  ! read variables
  ierr=nf_inq_varid(nid,'longitude',varid)
  ierr=nf_get_var_real(nid,varid,lon)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading longitude'
  endif
  ierr=nf_inq_varid(nid,'latitude',varid)
  ierr=nf_get_var_real(nid,varid,lat)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading latitude'
  endif
  ierr=nf_inq_varid(nid,'time',varid)
  ierr=nf_get_var_real(nid,varid,time)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading time'
  endif
  ierr=nf_inq_varid(nid,'emis_prior',varid)
  ierr=nf_get_var_real(nid,varid,emisprior)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading emis_prior'
  endif

  ierr=nf_close(nid)

  ! multiply by numscale and outheight
  emisprior=emisprior*numscale/outheight

  !---------------------------------------------------------------
  ! posterior error covariance
  !---------------------------------------------------------------

  ! open netcdf file
  write(logid,*) 'Reading cova.nc'
  ierr=nf_open(trim(dirinvert)//'cova.nc',NF_NOWRITE,nid)
  if(ierr.ne.0) then
    write(logid,*) 'Error opening cova.nc'
  endif

  ! inquire about variables
  ierr=nf_inq_dimid(nid,'dim1',xid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: dim1'
  endif
  ierr=nf_inq_dimid(nid,'dim2',yid)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension: dim2'
  endif

  ! read dimension sizes
  ierr=nf_inq_dimlen(nid,xid,nx)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  ierr=nf_inq_dimlen(nid,yid,ny)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading dimension length'
  endif
  nvar=nbox*nemsteps

  allocate(cova(nx,nx))

  ! read variable
  ierr=nf_inq_varid(nid,'cova',varid)
  ierr=nf_get_var_real(nid,varid,cova)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading cova'
  endif

  ierr=nf_close(nid)

  !---------------------------------------------------------------
  ! posterior state vector
  !---------------------------------------------------------------

  allocate(x0(nvar))

  write(logid,*) 'Reading posterior.txt' 
  open(100,file=trim(dirinvert)//'posterior.txt',action='read',status='old',iostat=ierr)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading posterior.txt'
  endif
  do i=1,nvar
    read(100,*) x0(i)
  end do
  close(100)
  ! multiply by numscale and outheight
  x0=x0*numscale/outheight

  !---------------------------------------------------------------
  ! prior state vector
  !---------------------------------------------------------------

  allocate(xb(nvar))

  write(logid,*) 'Reading prior.txt'
  open(100,file=trim(dirinvert)//'prior.txt',action='read',status='old',iostat=ierr)
  if(ierr.ne.0) then
    write(logid,*) 'Error reading prior.txt'
  endif
  do i=1,nvar
    read(100,*) xb(i)
  end do
  close(100)
  ! multiply by numscale and outheight
  xb=xb*numscale/outheight

end subroutine readinput


