!---------------------------------------------------------------------------------------
! FLEXINVERT : regridemissions
!---------------------------------------------------------------------------------------
!  Copyright 2013, Rona Thompson
!
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!---------------------------------------------------------------------------------------
!> Regrids emissions (or emission errors) to the flexpart resolution using the prior 
!! mass distribution within each aggregate box 
!---------------------------------------------------------------------------------------

subroutine regridemissions(datapost,dataprior,dataout)

  use mod_var

  implicit none

  integer :: n,nt,ix,jy
  real :: outlat0,area_grid
  real, dimension(nbox*nemsteps) :: dataprior,datapost
  real, dimension(nxgrid,nygrid,nemsteps) :: massratio,dataout

  ! calculate ratio of mass of each grid cell to mass in box aggregate for the prior emissions
  ! and use this to scale the posterior emissions in each grid cell

  outlat0=lat(1)
  dataout(:,:,:)=0.

  do n=1,nemsteps
    do jy=1,nygrid
      do ix=1,nxgrid
        if(dataprior((n-1)*nbox+nbox_xy(ix,jy)).gt.0.and.scaleflx) then
          call gridarea(outlat0+(float(jy)-0.5)*dy,dy,dx,area_grid)
          massratio(ix,jy,n)=emisprior(ix,jy,n)*area_grid/&
                             &(dataprior((n-1)*nbox+nbox_xy(ix,jy))*area_box(nbox_xy(ix,jy)))
          dataout(ix,jy,n)=massratio(ix,jy,n)*datapost((n-1)*nbox+nbox_xy(ix,jy))*&
                             &area_box(nbox_xy(ix,jy))/area_grid
        else
          dataout(ix,jy,n)=datapost((n-1)*nbox+nbox_xy(ix,jy))
        endif
      end do
    end do
  end do

end subroutine regridemissions
